/* automatically generated from indicator-dialog.glade */
#ifdef __SUNPRO_C
#pragma align 4 (indicator_dialog_ui)
#endif
#ifdef __GNUC__
static const char indicator_dialog_ui[] __attribute__ ((__aligned__ (4))) =
#else
static const char indicator_dialog_ui[] =
#endif
{
  "<?xml version=\"1.0\" encoding=\"UTF-8\"?><interface><requires lib=\"gt"
  "k+\" version=\"3.18\"/><requires lib=\"libxfce4ui-2\" version=\"4.12\"/"
  "><object class=\"GtkImage\" id=\"image1\"><property name=\"visible\">Tr"
  "ue</property><property name=\"can_focus\">False</property><property nam"
  "e=\"stock\">gtk-clear</property></object><object class=\"GtkListStore\""
  " id=\"indicators-store\"><columns><column type=\"GdkPixbuf\"/><column t"
  "ype=\"gchararray\"/><column type=\"gboolean\"/><column type=\"gboolean\""
  "/><column type=\"gchararray\"/></columns></object><object class=\"XfceT"
  "itledDialog\" id=\"dialog\"><property name=\"can_focus\">False</propert"
  "y><property name=\"title\" translatable=\"yes\">Indicators</property><p"
  "roperty name=\"default_width\">425</property><property name=\"default_h"
  "eight\">525</property><property name=\"icon_name\">gtk-properties</prop"
  "erty><property name=\"type_hint\">normal</property><child internal-chil"
  "d=\"vbox\"><object class=\"GtkBox\" id=\"dialog-vbox2\"><property name="
  "\"visible\">True</property><property name=\"can_focus\">False</property"
  "><property name=\"orientation\">vertical</property><property name=\"spa"
  "cing\">2</property><child internal-child=\"action_area\"><object class="
  "\"GtkButtonBox\" id=\"dialog-action_area2\"><property name=\"visible\">"
  "True</property><property name=\"can_focus\">False</property><property n"
  "ame=\"layout_style\">end</property><child><object class=\"GtkButton\" i"
  "d=\"close-button\"><property name=\"label\">gtk-close</property><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">True</p"
  "roperty><property name=\"receives_default\">True</property><property na"
  "me=\"use_stock\">True</property></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">False</property><property "
  "name=\"position\">0</property></packing></child><child><object class=\""
  "GtkButton\" id=\"help-button\"><property name=\"label\">gtk-help</prope"
  "rty><property name=\"visible\">True</property><property name=\"can_focu"
  "s\">True</property><property name=\"receives_default\">True</property><"
  "property name=\"use_stock\">True</property></object><packing><property "
  "name=\"expand\">False</property><property name=\"fill\">False</property"
  "><property name=\"position\">1</property><property name=\"secondary\">T"
  "rue</property></packing></child></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">True</property><property n"
  "ame=\"pack_type\">end</property><property name=\"position\">0</property"
  "></packing></child><child><object class=\"GtkBox\" id=\"vbox1\"><proper"
  "ty name=\"visible\">True</property><property name=\"can_focus\">False</"
  "property><property name=\"border_width\">12</property><property name=\""
  "orientation\">vertical</property><property name=\"spacing\">18</propert"
  "y><child><object class=\"GtkFrame\" id=\"frame3\"><property name=\"visi"
  "ble\">True</property><property name=\"can_focus\">False</property><prop"
  "erty name=\"label_xalign\">0</property><property name=\"shadow_type\">n"
  "one</property><child><object class=\"GtkAlignment\" id=\"alignment3\"><"
  "property name=\"visible\">True</property><property name=\"can_focus\">F"
  "alse</property><property name=\"top_padding\">6</property><property nam"
  "e=\"left_padding\">12</property><child><object class=\"GtkBox\" id=\"vb"
  "ox2\"><property name=\"visible\">True</property><property name=\"can_fo"
  "cus\">False</property><property name=\"orientation\">vertical</property"
  "><property name=\"spacing\">6</property><child><object class=\"GtkCheck"
  "Button\" id=\"checkbutton-single-row\"><property name=\"label\" transla"
  "table=\"yes\">Arrange indicators in a single row</property><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">True</proper"
  "ty><property name=\"receives_default\">False</property><property name=\""
  "tooltip_text\" translatable=\"yes\">If enabled, ensure that the indicat"
  "ors are laid out in a single row or column.</property><property name=\""
  "draw_indicator\">True</property></object><packing><property name=\"expa"
  "nd\">True</property><property name=\"fill\">True</property><property na"
  "me=\"position\">0</property></packing></child><child><object class=\"Gt"
  "kCheckButton\" id=\"checkbutton-align-left\"><property name=\"label\" t"
  "ranslatable=\"yes\">Align left in deskbar mode</property><property name"
  "=\"visible\">True</property><property name=\"can_focus\">True</property"
  "><property name=\"receives_default\">False</property><property name=\"t"
  "ooltip_text\" translatable=\"yes\">Controls the indicator button layout"
  " when the panel is in a Deskbar mode. Possible choices are \"centered\""
  " or \"aligned left\".</property><property name=\"draw_indicator\">True<"
  "/property></object><packing><property name=\"expand\">True</property><p"
  "roperty name=\"fill\">True</property><property name=\"position\">1</pro"
  "perty></packing></child><child><object class=\"GtkCheckButton\" id=\"ch"
  "eckbutton-square-icons\"><property name=\"label\" translatable=\"yes\">"
  "Square icons</property><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">True</property><property name=\"receives_default"
  "\">False</property><property name=\"tooltip_text\" translatable=\"yes\""
  ">Indicator buttons will take a square when it\'s possible.</property><p"
  "roperty name=\"draw_indicator\">True</property></object><packing><prope"
  "rty name=\"expand\">True</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">2</property></packing></child></object><"
  "/child></object></child><child type=\"label\"><object class=\"GtkLabel\""
  " id=\"label1\"><property name=\"visible\">True</property><property name"
  "=\"can_focus\">False</property><property name=\"label\" translatable=\""
  "yes\">Appearance</property><attributes><attribute name=\"weight\" value"
  "=\"bold\"/></attributes></object></child></object><packing><property na"
  "me=\"expand\">False</property><property name=\"fill\">True</property><p"
  "roperty name=\"position\">0</property></packing></child><child><object "
  "class=\"GtkFrame\" id=\"frame1\"><property name=\"visible\">True</prope"
  "rty><property name=\"can_focus\">False</property><property name=\"label"
  "_xalign\">0</property><property name=\"shadow_type\">none</property><ch"
  "ild><object class=\"GtkAlignment\" id=\"alignment1\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"top_padding\">6</property><property name=\"left_padding\""
  ">12</property><child><object class=\"GtkBox\" id=\"vbox3\"><property na"
  "me=\"visible\">True</property><property name=\"can_focus\">False</prope"
  "rty><property name=\"orientation\">vertical</property><property name=\""
  "spacing\">6</property><child><object class=\"GtkCheckButton\" id=\"chec"
  "kbutton-whitelist\"><property name=\"label\" translatable=\"yes\">Hide "
  "indicators by default</property><property name=\"visible\">True</proper"
  "ty><property name=\"can_focus\">True</property><property name=\"receive"
  "s_default\">False</property><property name=\"tooltip_text\" translatabl"
  "e=\"yes\">When enabled, only indicators marked \"Visible\" are shown. O"
  "therwise, all indicators not marked \"Hidden\" are displayed.</property"
  "><property name=\"draw_indicator\">True</property></object><packing><pr"
  "operty name=\"expand\">False</property><property name=\"fill\">True</pr"
  "operty><property name=\"position\">0</property></packing></child><child"
  "><object class=\"GtkBox\" id=\"hbox2\"><property name=\"visible\">True<"
  "/property><property name=\"can_focus\">False</property><property name=\""
  "spacing\">6</property><child><object class=\"GtkScrolledWindow\" id=\"s"
  "crolledwindow1\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"hscrollbar_policy\">ne"
  "ver</property><property name=\"shadow_type\">in</property><child><objec"
  "t class=\"GtkTreeView\" id=\"indicators-treeview\"><property name=\"vis"
  "ible\">True</property><property name=\"can_focus\">True</property><prop"
  "erty name=\"model\">indicators-store</property><property name=\"headers"
  "_clickable\">False</property><property name=\"rules_hint\">True</proper"
  "ty><property name=\"enable_search\">False</property><child internal-chi"
  "ld=\"selection\"><object class=\"GtkTreeSelection\"/></child><child><ob"
  "ject class=\"GtkTreeViewColumn\" id=\"treeviewcolumn-icon\"><property n"
  "ame=\"min_width\">24</property><child><object class=\"GtkCellRendererPi"
  "xbuf\" id=\"cellrendererpixbuf1\"/><attributes><attribute name=\"pixbuf"
  "\">0</attribute></attributes></child></object></child><child><object cl"
  "ass=\"GtkTreeViewColumn\" id=\"treeviewcolumn-name\"><property name=\"t"
  "itle\" translatable=\"yes\">Indicator</property><property name=\"expand"
  "\">True</property><child><object class=\"GtkCellRendererText\" id=\"cel"
  "lrenderertext1\"/><attributes><attribute name=\"text\">1</attribute></a"
  "ttributes></child></object></child><child><object class=\"GtkTreeViewCo"
  "lumn\" id=\"treeviewcolumn-blacklist\"><property name=\"title\" transla"
  "table=\"yes\">Hidden</property><child><object class=\"GtkCellRendererTo"
  "ggle\" id=\"hidden-toggle\"/><attributes><attribute name=\"active\">2</"
  "attribute></attributes></child></object></child><child><object class=\""
  "GtkTreeViewColumn\" id=\"treeviewcolumn-whitelist\"><property name=\"ti"
  "tle\" translatable=\"yes\">Visible</property><child><object class=\"Gtk"
  "CellRendererToggle\" id=\"visible-toggle\"/><attributes><attribute name"
  "=\"active\">3</attribute></attributes></child></object></child></object"
  "></child></object><packing><property name=\"expand\">True</property><pr"
  "operty name=\"fill\">True</property><property name=\"position\">0</prop"
  "erty></packing></child><child><object class=\"GtkBox\" id=\"vbox4\"><pr"
  "operty name=\"visible\">True</property><property name=\"can_focus\">Fal"
  "se</property><property name=\"orientation\">vertical</property><propert"
  "y name=\"spacing\">6</property><child><object class=\"GtkButton\" id=\""
  "item-up\"><property name=\"visible\">True</property><property name=\"ca"
  "n_focus\">True</property><property name=\"receives_default\">True</prop"
  "erty><property name=\"tooltip_text\" translatable=\"yes\">Move the sele"
  "cted indicator one row up.</property><child><object class=\"GtkImage\" "
  "id=\"image2\"><property name=\"visible\">True</property><property name="
  "\"can_focus\">False</property><property name=\"stock\">gtk-go-up</prope"
  "rty></object></child></object><packing><property name=\"expand\">False<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">0</property></packing></child><child><object class=\"GtkButton\" "
  "id=\"item-down\"><property name=\"visible\">True</property><property na"
  "me=\"can_focus\">True</property><property name=\"receives_default\">Tru"
  "e</property><property name=\"tooltip_text\" translatable=\"yes\">Move t"
  "he selected indicator one row down.</property><child><object class=\"Gt"
  "kImage\" id=\"image3\"><property name=\"visible\">True</property><prope"
  "rty name=\"can_focus\">False</property><property name=\"stock\">gtk-go-"
  "down</property></object></child></object><packing><property name=\"expa"
  "nd\">False</property><property name=\"fill\">True</property><property n"
  "ame=\"position\">1</property></packing></child></object><packing><prope"
  "rty name=\"expand\">False</property><property name=\"fill\">True</prope"
  "rty><property name=\"position\">1</property></packing></child></object>"
  "<packing><property name=\"expand\">True</property><property name=\"fill"
  "\">True</property><property name=\"position\">1</property></packing></c"
  "hild><child><object class=\"GtkLabel\" id=\"label4\"><property name=\"v"
  "isible\">True</property><property name=\"can_focus\">False</property><p"
  "roperty name=\"label\" translatable=\"yes\">&lt;i&gt;Please restart the"
  " panel for visibility changes to take effect.&lt;/i&gt;</property><prop"
  "erty name=\"use_markup\">True</property><property name=\"wrap\">True</p"
  "roperty><property name=\"xalign\">0</property></object><packing><proper"
  "ty name=\"expand\">False</property><property name=\"fill\">True</proper"
  "ty><property name=\"position\">2</property></packing></child><child><ob"
  "ject class=\"GtkButtonBox\" id=\"hbuttonbox1\"><property name=\"visible"
  "\">True</property><property name=\"can_focus\">False</property><propert"
  "y name=\"margin_top\">6</property><property name=\"layout_style\">start"
  "</property><child><object class=\"GtkButton\" id=\"indicators-clear\"><"
  "property name=\"label\" translatable=\"yes\">C_lear known indicators</p"
  "roperty><property name=\"visible\">True</property><property name=\"can_"
  "focus\">True</property><property name=\"receives_default\">True</proper"
  "ty><property name=\"tooltip_text\" translatable=\"yes\">Resets the list"
  " of indicators and their visibility settings.</property><property name="
  "\"image\">image1</property><property name=\"use_underline\">True</prope"
  "rty></object><packing><property name=\"expand\">False</property><proper"
  "ty name=\"fill\">False</property><property name=\"position\">0</propert"
  "y></packing></child></object><packing><property name=\"expand\">False</"
  "property><property name=\"fill\">True</property><property name=\"positi"
  "on\">3</property></packing></child></object></child></object></child><c"
  "hild type=\"label\"><object class=\"GtkLabel\" id=\"label3\"><property "
  "name=\"visible\">True</property><property name=\"can_focus\">False</pro"
  "perty><property name=\"label\" translatable=\"yes\">Known Indicators</p"
  "roperty><attributes><attribute name=\"weight\" value=\"bold\"/></attrib"
  "utes></object></child></object><packing><property name=\"expand\">True<"
  "/property><property name=\"fill\">True</property><property name=\"posit"
  "ion\">1</property></packing></child></object><packing><property name=\""
  "expand\">True</property><property name=\"fill\">True</property><propert"
  "y name=\"position\">1</property></packing></child></object></child><act"
  "ion-widgets><action-widget response=\"0\">close-button</action-widget><"
  "action-widget response=\"0\">help-button</action-widget></action-widget"
  "s></object><object class=\"GtkAdjustment\" id=\"size-adjustment\"><prop"
  "erty name=\"lower\">12</property><property name=\"upper\">64</property>"
  "<property name=\"value\">32</property><property name=\"step_increment\""
  ">1</property><property name=\"page_increment\">2</property></object></i"
  "nterface>"
};

static const unsigned indicator_dialog_ui_length = 13342u;

